@@ -16,8 +16,8 @@ import Yesod.Default.Config --(fromArgs)
 import Settings            --  (parseExtra)
 import Application          (makeApplication)
 import Data.String
-import Network.Wai.Handler.Warp (runSettings, defaultSettings, setPort)
-import Network.Wai.Handler.Launch (runUrlPort)
+import Network.Wai.Handler.Warp (runSettings, defaultSettings, setHost, setPort)
+import Network.Wai.Handler.Launch (runHostPortUrl)
 --
 #if !MIN_VERSION_base(4,8,0)
 import Control.Applicative ((<$>))
@@ -35,16 +35,19 @@ import Hledger.Cli hiding (progname,prognameandversion)
 import Hledger.Web.WebOptions
 
 
-main :: IO ()
-main = do
+hledgerWebMain :: IO ()
+hledgerWebMain = do
   opts <- getHledgerWebOpts
   when (debug_ (cliopts_ opts) > 0) $ printf "%s\n" prognameandversion >> printf "opts: %s\n" (show opts)
   runWith opts
 
 runWith :: WebOpts -> IO ()
 runWith opts
-  | "help" `inRawOpts` (rawopts_ $ cliopts_ opts)            = putStr (showModeHelp webmode) >> exitSuccess
-  | "version" `inRawOpts` (rawopts_ $ cliopts_ opts)         = putStrLn prognameandversion >> exitSuccess
+  | "h"               `inRawOpts` (rawopts_ $ cliopts_ opts) = putStr (showModeUsage webmode) >> exitSuccess
+  | "help"            `inRawOpts` (rawopts_ $ cliopts_ opts) = printHelpForTopic (topicForMode webmode) >> exitSuccess
+  | "man"             `inRawOpts` (rawopts_ $ cliopts_ opts) = runManForTopic (topicForMode webmode) >> exitSuccess
+  | "info"            `inRawOpts` (rawopts_ $ cliopts_ opts) = runInfoForTopic (topicForMode webmode) >> exitSuccess
+  | "version"         `inRawOpts` (rawopts_ $ cliopts_ opts) = putStrLn prognameandversion >> exitSuccess
   | "binary-filename" `inRawOpts` (rawopts_ $ cliopts_ opts) = putStrLn (binaryfilename progname)
   | otherwise = do
     requireJournalFileExists =<< (head `fmap` journalFilePathFromOpts (cliopts_ opts)) -- XXX head should be safe for now
@@ -57,7 +60,7 @@ withJournalDo' opts cmd = do
   -- https://github.com/simonmichael/hledger/issues/202
   -- -f- gives [Error#yesod-core] <stdin>: hGetContents: illegal operation (handle is closed) for some reason
   -- Also we may be writing to this file. Just disallow it.
-  when (f == "-") $ error' "hledger-web doesn't support --f -, please specify a file path"
+  when (f == "-") $ error' "hledger-web doesn't support -f -, please specify a file path"
 
   readJournalFile Nothing Nothing True f >>=
    either error' (cmd opts . journalApplyAliases (aliasesFromOpts $ cliopts_ opts))
@@ -66,24 +69,32 @@ withJournalDo' opts cmd = do
 web :: WebOpts -> Journal -> IO ()
 web opts j = do
   d <- getCurrentDay
-  let j' = filterJournalTransactions (queryFromOpts d $ reportopts_ $ cliopts_ opts) j
+  let initq = queryFromOpts d $ reportopts_ $ cliopts_ opts
+      j' = filterJournalTransactions initq j
+      h = "127.0.0.1"
       p = port_ opts
       u = base_url_ opts
       staticRoot = pack <$> file_url_ opts
-  _ <- printf "Starting web app on port %d with base url %s\n" p u
-  app <- makeApplication opts j' AppConfig{appEnv = Development
-                                          ,appPort = p
-                                          ,appRoot = pack u
-                                          ,appHost = fromString "*4"
-                                          ,appExtra = Extra "" Nothing staticRoot
-                                          }
+      appconfig = AppConfig{appEnv = Development
+                           ,appHost = fromString h
+                           ,appPort = p
+                           ,appRoot = pack u
+                           ,appExtra = Extra "" Nothing staticRoot
+                           }
+  app <- makeApplication opts j' appconfig
+  _ <- printf "Starting web app on host %s port %d with base url %s\n" h p u
   if server_ opts
-   then do
-    putStrLn "Press ctrl-c to quit"
-    hFlush stdout
-    Network.Wai.Handler.Warp.runSettings (setPort p defaultSettings) app
-   else do
-    putStrLn "Starting web browser if possible"
-    putStrLn "Web app will auto-exit after a few minutes with no browsers (or press ctrl-c)"
-    hFlush stdout
-    Network.Wai.Handler.Launch.runUrlPort p "" app
+    then do
+      putStrLn "Press ctrl-c to quit"
+      hFlush stdout
+      let warpsettings =
+            setHost (fromString h) $
+            setPort p $
+            defaultSettings
+      Network.Wai.Handler.Warp.runSettings warpsettings app
+    else do
+      putStrLn "Starting web browser..."
+      putStrLn "Web app will auto-exit after a few minutes with no browsers (or press ctrl-c)"
+      hFlush stdout
+      Network.Wai.Handler.Launch.runHostPortUrl h p "" app
+
